@@ -44,26 +44,26 @@ async def test_success():
     assert results == {'a', 'b'}
 
 
-@pytest.mark.parametrize('module, as_coro_obj', [
-    pytest.param(asyncio, True, marks=[
+@pytest.mark.parametrize('module', [
+    pytest.param(asyncio, id='asyncio', marks=[
         pytest.mark.skipif(not hasattr(asyncio, 'run'), reason='asyncio.run() is not available')]
     ),
-    (curio, False),
-    (trio, False)
-], ids=['asyncio', 'curio', 'trio'])
-def test_run_natively(module, as_coro_obj):
+    pytest.param(curio, id='curio'),
+    pytest.param(trio, id='trio')
+])
+def test_run_natively(module):
     async def testfunc():
         async with create_task_group() as tg:
             await tg.spawn(sleep, 0)
 
-    if as_coro_obj:
-        module.run(testfunc())
+    if module is asyncio:
+        try:
+            module.run(testfunc())
+        finally:
+            asyncio.set_event_loop(asyncio.new_event_loop())
     else:
         module.run(testfunc)
 
-    if module is asyncio:
-        asyncio.set_event_loop(asyncio.new_event_loop())
-
 
 @pytest.mark.anyio
 async def test_spawn_while_running():
